@@ -1,6 +1,7 @@
 import os
-from peewee import (Model, CharField, BooleanField, DateField, IntegerField,
-                    ForeignKeyField, SqliteDatabase)
+
+from peewee import (BooleanField, CharField, DateField, ForeignKeyField,
+                    IntegerField, Model, SqliteDatabase)
 from tmc.errors import NoCourseSelected, NoExerciseSelected
 
 sqlite = SqliteDatabase(
@@ -82,6 +83,10 @@ class Exercise(BaseModel):
             raise NoExerciseSelected()
         return first
 
+    @staticmethod
+    def byid(id):
+        return Exercise.get(Exercise.tid == int(id))
+
     def __str__(self):
         return "Exercise \"{}\" (ID {})".format(self.name, self.tid)
 
@@ -140,18 +145,16 @@ class Config(BaseModel):
             return False
 
 
-class DB(object):
+def init_db():
+    Course.create_table(fail_silently=True)
+    Exercise.create_table(fail_silently=True)
+    Config.create_table(fail_silently=True)
 
-    def __init__(self):
-        self.init()
+init_db()
 
-    def init(self):
-        Course.create_table(fail_silently=True)
-        Exercise.create_table(fail_silently=True)
-        Config.create_table(fail_silently=True)
 
-    def reset(self):
-        Course.drop_table()
-        Exercise.drop_table()
-        Config.drop_table()
-        self.init()
+def reset_db():
+    Course.drop_table()
+    Exercise.drop_table()
+    Config.drop_table()
+    init_db()
